@@ -32,7 +32,9 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "guiEngine.h"
 #include "player.h"
 #include "fontengine.h"
+#include "joystick_controller.h"
 #include "clientlauncher.h"
+#include "version.h"
 
 /* mainmenumanager.h
  */
@@ -40,31 +42,15 @@ gui::IGUIEnvironment *guienv = NULL;
 gui::IGUIStaticText *guiroot = NULL;
 MainMenuManager g_menumgr;
 
-bool noMenuActive()
+bool isMenuActive()
 {
-	return g_menumgr.menuCount() == 0;
+	return g_menumgr.menuCount() != 0;
 }
 
 // Passed to menus to allow disconnecting and exiting
 MainGameCallback *g_gamecallback = NULL;
 
 
-// Instance of the time getter
-static TimeGetter *g_timegetter = NULL;
-
-u32 getTimeMs()
-{
-	if (g_timegetter == NULL)
-		return 0;
-	return g_timegetter->getTime(PRECISION_MILLI);
-}
-
-u32 getTime(TimePrecision prec) {
-	if (g_timegetter == NULL)
-		return 0;
-	return g_timegetter->getTime(prec);
-}
-
 ClientLauncher::~ClientLauncher()
 {
 	if (receiver)
@@ -94,9 +80,6 @@ bool ClientLauncher::run(GameParams &game_params, const Settings &cmd_args)
 		return false;
 	}
 
-	// Create time getter
-	g_timegetter = new IrrlichtTimeGetter(device);
-
 	// Speed tests (done after irrlicht is loaded to get timer)
 	if (cmd_args.getFlag("speedtests")) {
 		dstream << "Running speed tests" << std::endl;
@@ -112,6 +95,8 @@ bool ClientLauncher::run(GameParams &game_params, const Settings &cmd_args)
 
 	porting::setXorgClassHint(video_driver->getExposedVideoData(), PROJECT_NAME_C);
 
+	porting::setWindowIcon(device);
+
 	/*
 		This changes the minimum allowed number of vertices in a VBO.
 		Default is 500.
@@ -123,10 +108,7 @@ bool ClientLauncher::run(GameParams &game_params, const Settings &cmd_args)
 
 	device->setResizable(true);
 
-	if (random_input)
-		input = new RandomInputHandler();
-	else
-		input = new RealInputHandler(device, receiver);
+	init_input();
 
 	smgr = device->getSceneManager();
 	smgr->getParameters()->setAttribute(scene::ALLOW_ZWRITE_ON_TRANSPARENT, true);
@@ -184,7 +166,9 @@ bool ClientLauncher::run(GameParams &game_params, const Settings &cmd_args)
 	{
 		// Set the window caption
 		const wchar_t *text = wgettext("Main Menu");
-		device->setWindowCaption((utf8_to_wide(PROJECT_NAME_C) + L" [" + text + L"]").c_str());
+		device->setWindowCaption((utf8_to_wide(PROJECT_NAME_C) +
+			L" " + utf8_to_wide(g_version_hash) +
+			L" [" + text + L"]").c_str());
 		delete[] text;
 
 		try {	// This is used for catching disconnects
@@ -331,6 +315,33 @@ bool ClientLauncher::init_engine()
 	return device != NULL;
 }
 
+void ClientLauncher::init_input()
+{
+	if (random_input)
+		input = new RandomInputHandler();
+	else
+		input = new RealInputHandler(device, receiver);
+
+	if (g_settings->getBool("enable_joysticks")) {
+		irr::core::array<irr::SJoystickInfo> infos;
+		std::vector<irr::SJoystickInfo> joystick_infos;
+
+		// Make sure this is called maximum once per
+		// irrlicht device, otherwise it will give you
+		// multiple events for the same joystick.
+		if (device->activateJoysticks(infos)) {
+			infostream << "Joystick support enabled" << std::endl;
+			joystick_infos.reserve(infos.size());
+			for (u32 i = 0; i < infos.size(); i++) {
+				joystick_infos.push_back(infos[i]);
+			}
+			input->joystick.onJoystickConnect(joystick_infos);
+		} else {
+			errorstream << "Could not activate joystick support." << std::endl;
+		}
+	}
+}
+
 bool ClientLauncher::launch_game(std::string &error_message,
 		bool reconnect_requested, GameParams &game_params,
 		const Settings &cmd_args)
@@ -397,15 +408,14 @@ bool ClientLauncher::launch_game(std::string &error_message,
 		return false;
 	}
 
-	if (menudata.name == "")
-		menudata.name = std::string("Guest") + itos(myrand_range(1000, 9999));
-	else
-		playername = menudata.name;
+	if (menudata.name == "" && !simple_singleplayer_mode) {
+		error_message = gettext("Please choose a name!");
+		return false;
+	}
 
+	playername = menudata.name;
 	password = menudata.password;
 
-	g_settings->set("name", playername);
-
 	current_playername = playername;
 	current_password   = password;
 	current_address    = address;
@@ -418,13 +428,16 @@ bool ClientLauncher::launch_game(std::string &error_message,
 		current_password = "";
 		current_address = "";
 		current_port = myrand_range(49152, 65535);
-	} else if (address != "") {
-		ServerListSpec server;
-		server["name"] = menudata.servername;
-		server["address"] = menudata.address;
-		server["port"] = menudata.port;
-		server["description"] = menudata.serverdescription;
-		ServerList::insert(server);
+	} else {
+		g_settings->set("name", playername);
+		if (address != "") {
+			ServerListSpec server;
+			server["name"] = menudata.servername;
+			server["address"] = menudata.address;
+			server["port"] = menudata.port;
+			server["description"] = menudata.serverdescription;
+			ServerList::insert(server);
+		}
 	}
 
 	infostream << "Selected world: " << worldspec.name
@@ -483,7 +496,7 @@ void ClientLauncher::main_menu(MainMenuData *menudata)
 
 	infostream << "Waiting for other menus" << std::endl;
 	while (device->run() && *kill == false) {
-		if (noMenuActive())
+		if (!isMenuActive())
 			break;
 		driver->beginScene(true, true, video::SColor(255, 128, 128, 128));
 		guienv->drawAll();
@@ -499,7 +512,8 @@ void ClientLauncher::main_menu(MainMenuData *menudata)
 #endif
 
 	/* show main menu */
-	GUIEngine mymenu(device, guiroot, &g_menumgr, smgr, menudata, *kill);
+	GUIEngine mymenu(device, &input->joystick, guiroot,
+		&g_menumgr, smgr, menudata, *kill);
 
 	smgr->clear();	/* leave scene manager in a clean state */
 }
@@ -521,7 +535,7 @@ bool ClientLauncher::create_engine_device()
 
 	// Determine driver
 	video::E_DRIVER_TYPE driverType = video::EDT_OPENGL;
-	std::string driverstring = g_settings->get("video_driver");
+	const std::string &driverstring = g_settings->get("video_driver");
 	std::vector<video::E_DRIVER_TYPE> drivers
 		= porting::getSupportedVideoDrivers();
 	u32 i;
@@ -557,9 +571,8 @@ bool ClientLauncher::create_engine_device()
 
 	device = createDeviceEx(params);
 
-	if (device) {
+	if (device)
 		porting::initIrrlicht(device);
-	}
 
 	return device != NULL;
 }
