@@ -37,6 +37,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "log.h"
 #include "fontengine.h"
 #include "guiscalingfilter.h"
+#include "irrlicht_changes/static_text.h"
 
 #ifdef __ANDROID__
 #include "client/tile.h"
@@ -59,7 +60,7 @@ void TextDestGuiEngine::gotText(const StringMap &fields)
 }
 
 /******************************************************************************/
-void TextDestGuiEngine::gotText(std::wstring text)
+void TextDestGuiEngine::gotText(const std::wstring &text)
 {
 	m_engine->getScriptIface()->handleMainMenuEvent(wide_to_utf8(text));
 }
@@ -130,6 +131,7 @@ void MenuMusicFetcher::fetchSounds(const std::string &name,
 /** GUIEngine                                                                 */
 /******************************************************************************/
 GUIEngine::GUIEngine(	irr::IrrlichtDevice* dev,
+						JoystickController *joystick,
 						gui::IGUIElement* parent,
 						IMenuManager *menumgr,
 						scene::ISceneManager* smgr,
@@ -172,30 +174,29 @@ GUIEngine::GUIEngine(	irr::IrrlichtDevice* dev,
 		m_sound_manager = &dummySoundManager;
 
 	//create topleft header
-	std::wstring t = utf8_to_wide(std::string(PROJECT_NAME_C " ") +
-			g_version_hash);
+	m_toplefttext = L"";
 
-	core::rect<s32> rect(0, 0, g_fontengine->getTextWidth(t), g_fontengine->getTextHeight());
+	core::rect<s32> rect(0, 0, g_fontengine->getTextWidth(m_toplefttext.c_str()),
+		g_fontengine->getTextHeight());
 	rect += v2s32(4, 0);
 
 	m_irr_toplefttext =
-		m_device->getGUIEnvironment()->addStaticText(t.c_str(),
-		rect,false,true,0,-1);
+		addStaticText(m_device->getGUIEnvironment(), m_toplefttext,
+			rect, false, true, 0, -1);
 
 	//create formspecsource
 	m_formspecgui = new FormspecFormSource("");
 
 	/* Create menu */
 	m_menu = new GUIFormSpecMenu(m_device,
+			joystick,
 			m_parent,
 			-1,
 			m_menumanager,
 			NULL /* &client */,
-			NULL /* gamedef */,
 			m_texture_source,
 			m_formspecgui,
 			m_buttonhandler,
-			NULL,
 			false);
 
 	m_menu->allowClose(false);
@@ -212,13 +213,13 @@ GUIEngine::GUIEngine(	irr::IrrlichtDevice* dev,
 		m_data->script_data.errormessage = "";
 
 		if (!loadMainMenuScript()) {
-			errorstream << "No future without mainmenu" << std::endl;
+			errorstream << "No future without main menu!" << std::endl;
 			abort();
 		}
 
 		run();
 	} catch (LuaError &e) {
-		errorstream << "MAINMENU ERROR: " << e.what() << std::endl;
+		errorstream << "Main menu error: " << e.what() << std::endl;
 		m_data->script_data.errormessage = e.what();
 	}
 
@@ -230,13 +231,13 @@ GUIEngine::GUIEngine(	irr::IrrlichtDevice* dev,
 /******************************************************************************/
 bool GUIEngine::loadMainMenuScript()
 {
-	// Try custom menu script (main_menu_path)
-
+	// Set main menu path (for core.get_mainmenu_path())
 	m_scriptdir = g_settings->get("main_menu_path");
 	if (m_scriptdir.empty()) {
-		m_scriptdir = porting::path_share + DIR_DELIM "builtin" + DIR_DELIM "mainmenu";
+		m_scriptdir = porting::path_share + DIR_DELIM + "builtin" + DIR_DELIM + "mainmenu";
 	}
 
+	// Load builtin (which will load the main menu script)
 	std::string script = porting::path_share + DIR_DELIM "builtin" + DIR_DELIM "init.lua";
 	try {
 		m_script->loadScript(script);
@@ -261,8 +262,24 @@ void GUIEngine::run()
 
 	unsigned int text_height = g_fontengine->getTextHeight();
 
-	while(m_device->run() && (!m_startgame) && (!m_kill))
-	{
+	irr::core::dimension2d<u32> previous_screen_size(g_settings->getU16("screenW"),
+		g_settings->getU16("screenH"));
+
+	while (m_device->run() && (!m_startgame) && (!m_kill)) {
+
+		const irr::core::dimension2d<u32> &current_screen_size =
+			m_device->getVideoDriver()->getScreenSize();
+		// Verify if window size has changed and save it if it's the case
+		// Ensure evaluating settings->getBool after verifying screensize
+		// First condition is cheaper
+		if (previous_screen_size != current_screen_size &&
+				current_screen_size != irr::core::dimension2d<u32>(0,0) &&
+				g_settings->getBool("autosave_screensize")) {
+			g_settings->setU16("screenW", current_screen_size.Width);
+			g_settings->setU16("screenH", current_screen_size.Height);
+			previous_screen_size = current_screen_size;
+		}
+
 		//check if we need to update the "upper left corner"-text
 		if (text_height != g_fontengine->getTextHeight()) {
 			updateTopLeftTextSize();
@@ -430,7 +447,7 @@ void GUIEngine::drawOverlay(video::IVideoDriver* driver)
 
 	video::ITexture* texture = m_textures[TEX_LAYER_OVERLAY].texture;
 
-	/* If no texture, draw background of solid color */
+	/* If no texture, draw nothing */
 	if(!texture)
 		return;
 
@@ -459,7 +476,7 @@ void GUIEngine::drawHeader(video::IVideoDriver* driver)
 	v2s32 splashsize(((f32)texture->getOriginalSize().Width) * mult,
 			((f32)texture->getOriginalSize().Height) * mult);
 
-	// Don't draw the header is there isn't enough room
+	// Don't draw the header if there isn't enough room
 	s32 free_space = (((s32)screensize.Height)-320)/2;
 
 	if (free_space > splashsize.Y) {
@@ -539,7 +556,7 @@ bool GUIEngine::setTexture(texture_layer layer, std::string texturepath,
 }
 
 /******************************************************************************/
-bool GUIEngine::downloadFile(std::string url, std::string target)
+bool GUIEngine::downloadFile(const std::string &url, const std::string &target)
 {
 #if USE_CURL
 	std::ofstream target_file(target.c_str(), std::ios::out | std::ios::binary);
@@ -567,18 +584,9 @@ bool GUIEngine::downloadFile(std::string url, std::string target)
 }
 
 /******************************************************************************/
-void GUIEngine::setTopleftText(std::string append)
+void GUIEngine::setTopleftText(const std::string &text)
 {
-	std::wstring toset = utf8_to_wide(std::string(PROJECT_NAME_C " ") +
-		g_version_hash);
-
-	if (append != "")
-	{
-		toset += L" / ";
-		toset += utf8_to_wide(append);
-	}
-
-	m_irr_toplefttext->setText(toset.c_str());
+	m_toplefttext = utf8_to_wide(text);
 
 	updateTopLeftTextSize();
 }
@@ -586,15 +594,14 @@ void GUIEngine::setTopleftText(std::string append)
 /******************************************************************************/
 void GUIEngine::updateTopLeftTextSize()
 {
-	std::wstring text = m_irr_toplefttext->getText();
-
-	core::rect<s32> rect(0, 0, g_fontengine->getTextWidth(text), g_fontengine->getTextHeight());
-		rect += v2s32(4, 0);
+	core::rect<s32> rect(0, 0, g_fontengine->getTextWidth(m_toplefttext.c_str()),
+		g_fontengine->getTextHeight());
+	rect += v2s32(4, 0);
 
 	m_irr_toplefttext->remove();
 	m_irr_toplefttext =
-		m_device->getGUIEnvironment()->addStaticText(text.c_str(),
-		rect,false,true,0,-1);
+		addStaticText(m_device->getGUIEnvironment(), m_toplefttext,
+			rect, false, true, 0, -1);
 }
 
 /******************************************************************************/
@@ -611,8 +618,8 @@ void GUIEngine::stopSound(s32 handle)
 }
 
 /******************************************************************************/
-unsigned int GUIEngine::queueAsync(std::string serialized_func,
-		std::string serialized_params)
+unsigned int GUIEngine::queueAsync(const std::string &serialized_func,
+		const std::string &serialized_params)
 {
 	return m_script->queueAsync(serialized_func, serialized_params);
 }
