@@ -44,7 +44,7 @@
 #if USE_SYSTEM_GMP || defined (__ANDROID__) || defined (ANDROID)
 	#include <gmp.h>
 #else
-	#include <gmp/mini-gmp.h>
+	#include <mini-gmp.h>
 #endif
 
 #include <util/sha2.h>
@@ -542,7 +542,7 @@ static SRP_Result fill_buff()
 
 	if (!fp) return SRP_ERR;
 
-	if (fread(g_rand_buff, sizeof(g_rand_buff), 1, fp) != 1) return SRP_ERR;
+	if (fread(g_rand_buff, sizeof(g_rand_buff), 1, fp) != 1) { fclose(fp); return SRP_ERR; }
 	if (fclose(fp)) return SRP_ERR;
 #endif
 	return SRP_OK;
